import bpy 
import math


    
def G_rig():
    
    
    mesh_object = bpy.context.active_object
    
    # Lista para almacenar las posiciones de los vértices y sus normales
    points_positions = []
    vertex_normals = []

    # Verifica que el objeto seleccionado sea una malla
    if mesh_object and mesh_object.type == 'MESH':
        # Matriz mundial del objeto de malla
        global_matrix = mesh_object.matrix_world

        # Itera sobre los vértices de la malla
        for vertex in mesh_object.data.vertices:
            # Transforma la posición del vértice a coordenadas globales
            global_position = global_matrix @ vertex.co
            points_positions.append(global_position)

            # Obtiene la normal del vértice y la transforma a coordenadas globales
            global_normal = global_matrix @ vertex.normal
            vertex_normals.append(global_normal)
        
        modificador = mesh_object.modifiers.new("GENERADOR", 'SKIN')
        
        if modificador:
            bpy.ops.object.skin_armature_create(modifier="Skin")
            mesh_object.modifiers.remove(modificador)
    
    #add Armadura
    Armadura = bpy.context.active_object
    if Armadura:
        
        Armadura.data.name = "G_Rig_Mesh"
        threshold_distance = 0.001 
        
        bpy.ops.object.mode_set(mode='EDIT')
        
        def distancia(point1, point2):
            return math.sqrt(sum((a - b) ** 2 for a, b in zip(point1, point2)))
        
        def eliminar_huesos_similares(armadura, distancia_umbral):
            # Asegúrate de estar en el modo correcto
            bpy.context.view_layer.objects.active = armadura
            huesos = armadura.data.edit_bones
            huesos_a_eliminar = set()

            # Convertir huesos a una lista para evitar modificar la colección mientras se itera
            lista_huesos = list(huesos)

            for i, H in enumerate(lista_huesos):
                if H.name in huesos_a_eliminar:
                    continue
                for j in range(i + 1, len(lista_huesos)):
                    h = lista_huesos[j]
                    if h.name in huesos_a_eliminar:
                        continue
                    distancia_head = distancia(H.head, h.head)
                    distancia_tail = distancia(H.tail, h.tail)

                    if distancia_head < distancia_umbral and distancia_tail < distancia_umbral:
                        huesos_a_eliminar.add(h.name)  # Marcar el segundo hueso para eliminación
                        break  # No es necesario comprobar otros huesos si decidimos eliminar este

            # Eliminar huesos después de las comprobaciones para evitar modificar la colección durante la iteración
            for nombre_hueso in huesos_a_eliminar:
                if nombre_hueso in huesos:
                    huesos.remove(huesos[nombre_hueso])  
        
        bones = Armadura.data.edit_bones
        DEF_BONES = []
        
        for B in bones:
            eliminar_huesos_similares(Armadura, threshold_distance)
            
            B.name = "B.V_DEF.001"
            DEF_BONES.append(B)
            
        BONES = []
        
        for i, point_position in enumerate(points_positions):
            bone = Armadura.data.edit_bones.new(f"B.V_BASE{i+1:03}")
            
            BONES.append(bone)
            
            # Asigna la posición del vértice al hueso
            bone.head = point_position
            
            # Calcula la cola (tail) del hueso usando la normal del vértice
            # Se multiplicará la normal por una cierta longitud para determinar la posición de la cola
            tail_position = point_position + 0.5 * vertex_normals[i]  # Aquí 0.5 es la longitud del hueso
            bone.tail = tail_position
            bone.use_deform = False

            
        for i in range(len(DEF_BONES)):
            for b in BONES:
                distance = math.sqrt(sum((a - b) ** 2 for a, b in zip(DEF_BONES[i].head, b.head)))
                if distance < threshold_distance:
                    DEF_BONES[i].use_connect = False
                    DEF_BONES[i].parent = b
                    
