import bpy 
import time
import sys
import os

current_dir = os.path.dirname(__file__)
sys.path.append(current_dir) 

class enum_items_list_(bpy.types.UIList):
    
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(item, "name",text="",emboss=False)
            layout.prop(item, "type",text="",emboss=False)
            
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="")



class Group_Enum_list(bpy.types.PropertyGroup):
    type: bpy.props.StringProperty()
    name: bpy.props.StringProperty()
    description: bpy.props.StringProperty()
    index: bpy.props.IntProperty()
    icon: bpy.props.StringProperty()
    
 

class Group_Enum_(bpy.types.PropertyGroup):
    
    items: bpy.props.CollectionProperty(type=Group_Enum_list)
    index: bpy.props.IntProperty()
    
    def items_(self, context):
        Enum_items = []
        for test_number in self.items:
            item = (test_number.type, test_number.name, test_number.description)
            Enum_items.append(item)
        return Enum_items or [("NONE", "None", "No items available")] 
    
    Enum: bpy.props.EnumProperty(items=items_)
    name: bpy.props.StringProperty()
    
    use_visible: bpy.props.BoolProperty(default=False) 
    
class Enum_Custom_Properties(bpy.types.Panel):
    bl_idname = "ENUM_PT_custom_properties"
    bl_label = "Enum Custom Properties"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "object"
    bl_parent_id = "OBJECT_PT_custom_props"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout
        
        props = context.active_object.Enum
        layout.operator("add.enum_prop", text="New", icon="ADD").type = "ADD_ENUM"
        for index, enum in enumerate(props):
            row = layout.row()
            row.alignment = "RIGHT"
            row.label(text=enum.name)
            if enum.use_visible:
                row_ = row.row(align=True)
                for item in enum.items:
                    row_.prop_enum(enum,"Enum",f"{item.type}",text=item.name) 
            else:
                row.prop(enum, "Enum", text="")

            new = row.operator("add.enum_prop", text="", icon="PREFERENCES", emboss=False)
            new.type = "CALL_PANEL"
            new.index = index

            new = row.operator("add.enum_prop", text="", icon="X", emboss=False)
            new.type = "DELL"
            new.index = index
        

class edit_enum(bpy.types.Panel):
    bl_idname = "edit_enum"
    bl_label = "Edit Enum"
    bl_space_type = 'CLIP_EDITOR'  # Selecciona un espacio poco común
    bl_region_type = 'UI'  # Región común pero poco intrusiva
    bl_options = {'DEFAULT_CLOSED'}
    
    
    def draw(self, context):
        layout = self.layout
        obj = context.active_object
        enums = obj.Enum
        
        index = obj.enum_edit_index
        if index < len(enums):
            enum = enums[index]
            layout.prop(enum, "name", text="Label")
            row = layout.row()
            row.template_list(
                "enum_items_list_", 
                "", 
                enum, "items", 
                enum, "index",
                rows=4 if enum.items else 1,)
            
            col = row.column(align=True)
            col.operator("add.enum_prop", text="", icon="ADD").type = "ADD_ITEMS"
            col_= col.column(align=True)
            col_.enabled = len(enum.items) > 0
            col_.operator("add.enum_prop", text="", icon="REMOVE").type = "REMOVE_ITEMS"

            if enum.items: 
                col = row.column(align=True)
                col_.operator("add.enum_prop", text="", icon="TRIA_UP").type = "MOVE_UP_ITEMS"
                col_.operator("add.enum_prop", text="", icon="TRIA_DOWN").type = "MOVE_DOWN_ITEMS"
            
                layout.prop(enum.items[enum.index], "name", text="Name")
                layout.prop(enum.items[enum.index], "type", text="Type")
                layout.prop(enum.items[enum.index], "description", text="Description")
                layout.prop(enum, "use_visible", text="Select View")
                
class ADD_ENUM_PROP_OT_add_enum_prop(bpy.types.Operator):
    bl_idname = "add.enum_prop"
    bl_label = "Add Enum Property"
    bl_description = "Add a new Enum property"
    bl_options = {"REGISTER"}

    type: bpy.props.StringProperty()
    
    index: bpy.props.IntProperty()
    
    def execute(self, context):
        obj = context.object
        
        def move(delta):
            enums = obj.Enum
            index = obj.enum_edit_index
            if index < len(enums):
                enum = enums[index]
                
                idx = enum.index
                # Ensure idx is within valid bounds
                if idx > len(enum.items) - 1:
                    idx = len(enum.items) - 1
                elif idx < 0:
                    idx = 0

                new_idx = idx + delta

                # Ensure new_idx is within valid bounds
                if new_idx < 0:
                    new_idx = 0
                elif new_idx > len(enum.items) - 1:
                    new_idx = len(enum.items) - 1

                enum.items.move(idx, new_idx)
                enum.index = new_idx
        
        
        if self.type == "ADD_ENUM":
            new = obj.Enum.add()
            new.name = "Enum"
        elif self.type == "DELL":
            obj.Enum.remove(self.index)
            
        elif self.type == "CALL_PANEL":
            obj.enum_edit_index = self.index
            bpy.ops.wm.call_panel(name=edit_enum.bl_idname)
            
        elif self.type == "ADD_ITEMS":
            enums = obj.Enum
            index = obj.enum_edit_index
            if index < len(enums):
                enum = enums[index]
                new = enum.items.add()
                new.type = "NONE"
                new.name = "None"
                enum.index = len(enum.items)-1
        
        
                
        elif self.type == "MOVE_UP_ITEMS":
            move(-1)
        elif self.type == "MOVE_DOWN_ITEMS":
            move(1)    
                
        elif self.type == "REMOVE_ITEMS":
            
            enums = obj.Enum
            index = obj.enum_edit_index
            if index < len(enums):
                enum = enums[index]
                enum.items.remove(enum.index)     
            enum.index = min(max(0, enum.index - 1), len(enum.items) - 1)
        return {"FINISHED"}


classes = (
    enum_items_list_,
    edit_enum,
    Group_Enum_list,
    Group_Enum_,
    Enum_Custom_Properties,
    ADD_ENUM_PROP_OT_add_enum_prop,
)

def properties_enum_set(self, context):
    layout = self.layout
    props = context.active_object.Enum
    for index, enum in enumerate(props):
        row = layout.row()
        row.label(text=enum.name)
        if enum.use_visible:
            row_ = row.row(align=True)
            for item in enum.items:
                row_.prop_enum(enum,"Enum",f"{item.type}",text=item.name) 
        else:
            row.prop(enum, "Enum", text="")   
        
def regenum():
    for cls in classes:
        bpy.utils.register_class(cls)
    
    # Register Enum properties
    bpy.types.Object.enum_edit_index = bpy.props.IntProperty()
    bpy.types.Object.Enum = bpy.props.CollectionProperty(type=Group_Enum_) 
    bpy.types.VIEW3D_PT_context_properties.append(properties_enum_set)

    
def unregenum():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    
    del bpy.types.Object.enum_edit_index    
    del bpy.types.Object.Enum
    bpy.types.VIEW3D_PT_context_properties.remove(properties_enum_set)
