import bpy 
import time
import sys
import os
from bl_ui.generic_ui_list import draw_ui_list
current_dir = os.path.dirname(__file__)
sys.path.append(current_dir) 

from Operadores.Rig_Creator_Operation.RIg_Constraint_G import *
from Paneles.Paneles_Constraint.Constraint import *

class BONE_CONST_PropertyGroup(bpy.types.PropertyGroup):
    
    name: bpy.props.StringProperty(default = "")
    icon: bpy.props.StringProperty(default = "CONSTRAINT_BONE")
    type: bpy.props.StringProperty(default = "")
    
    enabled: bpy.props.BoolProperty(default=False)
    
    influence: bpy.props.FloatProperty(default=1.0, subtype='FACTOR', min=0.0, max=1.0)


class BONE_CONST_UL_List(bpy.types.UIList):
    
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            if context.active_pose_bone:
                bone = context.active_pose_bone
                for constraint in bone.constraints:
                    if constraint.name == item.name:
                        
                        row = layout.row(align=True)
                        row.alignment = 'LEFT'
                        row.scale_x = 1.1
                        row.prop(constraint, "name", text="", icon=item.icon,  emboss=False)
                        row = layout.row(align=True)
                        row.alignment = 'RIGHT'
                        row.prop(constraint, "influence", text="",  emboss=False)
                        row.prop(constraint, "enabled", text="",icon='HIDE_ON',  emboss=False)
                
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon=item.icon)
            
class BoneConstraintPanel:
    bl_context = "bone_constraint"

    @classmethod
    def poll(cls, context):
        return (context.pose_bone)     
        
class BONE_PT_constraints(BoneConstraintPanel,bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_label = "Bone Constraints"
    bl_options = {'HIDE_HEADER'}
        
    def draw(self, context):
        layout = self.layout
        
        if context.active_pose_bone:
            layout.active = True
        else:
            layout.active = False
            
        row = layout.row(align=True)
        
        row.operator_menu_enum("pose.constraint_add", "type", text="Add Bone Constraint")
        
        layout.template_constraints(use_bone_constraints=True)
        #row.scale_x = 0.5
        #row.menu("add_bone_const_Custom", text="Custom")
        
class Constraint_list(bpy.types.Panel):
    bl_idname = "Constraint_list"
    bl_label = "Constraint List"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "bone_constraint"
    bl_order = 0
    
    @classmethod
    def poll(cls, context):
        return (
            context.pose_bone and context.active_pose_bone
            )
    
    def draw(self, _context):
        layout = self.layout
        Constraint_list_Draw(self, _context, use_Bone=True)

class Copy_Constraint_To_Active(bpy.types.Menu):
    bl_idname = "Copy_Constraint_To_Active"
    bl_label = "Copy Constraint To Active"
    
    def draw(self, context):
        layout = self.layout
        Extra_Botones(self, context, use_Bone = True)
        
class add_bone_const_motion_tracking(bpy.types.Menu):
    bl_idname = "add_bone_const_motion_tracking"
    bl_label = "Motion Tracking"
    @classmethod
    def poll(cls, context):
        return (context.pose_bone and context.active_pose_bone)
    def draw(self, context):
        layout = self.layout
        layout.label(text="Motion Tracking")
        for i in range(3):
            op=layout.operator(
                "add.bone_const", 
                text=constraint_types[i][1], 
                icon=constraint_types[i][2]
                )
            op.type = constraint_types[i][0]
            op.index = 0
            op.is_Custom = ""
            
class add_bone_const_transform(bpy.types.Menu):
    bl_idname = "add_bone_const_transform"
    bl_label = "Transform"
    @classmethod
    def poll(cls, context):
        return (context.pose_bone and context.active_pose_bone)
    def draw(self, context):
        layout = self.layout
        layout.label(text="Transform")
        for i in range(11):
            op=layout.operator(
                "add.bone_const", 
                text=constraint_types[i+3][1], 
                icon=constraint_types[i+3][2]
                )
            op.type = constraint_types[i+3][0]
            op.index = 0
            op.is_Custom = ""
            
class add_bone_const_tracking(bpy.types.Menu):
    bl_idname = "add_bone_const_tracking"
    bl_label = "Tracking"
    @classmethod
    def poll(cls, context):
        return (context.pose_bone and context.active_pose_bone)
    def draw(self, context):
        layout = self.layout
        layout.label(text="Tracking")
        for i in range(7):
            op=layout.operator(
                "add.bone_const", 
                text=constraint_types[i+(3+11)][1], 
                icon=constraint_types[i+(3+11)][2]
                )
            op.type = constraint_types[i+(3+11)][0]
            op.index = 0
            op.is_Custom = ""
            
class add_bone_const_relationship(bpy.types.Menu):
    bl_idname = "add_bone_const_relationship"
    bl_label = "Relationship"
    @classmethod
    def poll(cls, context):
        return (context.pose_bone and context.active_pose_bone)
    def draw(self, context):
        layout = self.layout  
        layout.label(text="Relationship")
        for i in range(7):

            op =layout.operator(
                "add.bone_const", 
                text=constraint_types[i+(3+11+7)][1], 
                icon=constraint_types[i+(3+11+7)][2]
                )
            op.type = constraint_types[i+(3+11+7)][0]
            op.index = 0
            op.is_Custom = ""
            
class add_bone_const_Custom(bpy.types.Menu):
    bl_idname = "add_bone_const_Custom"
    bl_label = "Relationship"
    @classmethod
    def poll(cls, context):
        return (context.pose_bone and context.active_pose_bone)
    def draw(self, context):
        layout = self.layout  
        layout.label(text="Custom")
        
        save_constraint_index = bpy.context.scene.save_constraint_index
        save_const = bpy.context.scene.save_constraint
        
        for index, const_info in enumerate(save_const):
            op = layout.operator("add.bone_const", text=const_info["name"], icon= 'ADD')
            op.type = "" 
            op.index = index
            op.is_Custom = "CUSTOM"
            
class add_bone_const_operator(bpy.types.Operator):
    bl_idname = "add.bone_const"
    bl_label = "Add Bone Constraint"
    bl_description = "Description that shows in blender tooltips"
    bl_options = {"REGISTER"}
    
    is_Custom: bpy.props.StringProperty(default="")
    
    type: bpy.props.StringProperty(default="")
    
    index: bpy.props.IntProperty(default=1)

    def execute(self, context):
        
        save_const = bpy.context.scene.save_constraint
        
        if self.is_Custom != 'CUSTOM':
            Bone = bpy.context.active_pose_bone
            constraint = Bone.constraints.new(type=self.type)
        else:
            Rig_props = bpy.context.scene.Rig_Creator_PropertyGroup.Add_activo_or_Seletion
            bpy.context.scene.Rig_Creator_PropertyGroup.Add_activo_or_Seletion = 'ACTIVE'
            add_const_to_Bone(self.is_Custom, save_const[self.index])
            bpy.context.scene.Rig_Creator_PropertyGroup.Add_activo_or_Seletion = Rig_props
            
        return {"FINISHED"}
    
def copy_cosntraint(self, context, constraint_name, use_index=True):
    obj = context.object
        
    # Ensure we have an active armature object
    if obj is None or obj.type != 'ARMATURE':
        self.report({'ERROR'}, "No active armature object found")
        return {'CANCELLED'}
    
    # Ensure we have an active pose bone
    source_bone = context.active_pose_bone
    if not source_bone:
        self.report({'ERROR'}, "No active pose bone found")
        return {'CANCELLED'}
    
    # Find the constraint in the active bone
    constraint = None
    for const in source_bone.constraints:
        if const.name == constraint_name:
            constraint = const
            break
        
    if not constraint:
        self.report({'ERROR'}, f"Constraint '{constraint_name}' not found")
        return {'CANCELLED'}
    
    # Get the index of the constraint
    if use_index: 
        constraint_index = source_bone.constraints.find(constraint_name)
    
    # Copy the constraint to the selected bones
    for bone in context.selected_pose_bones:
        
        if bone != source_bone:
            new_constraint = bone.constraints.new(type=constraint.type)
            
            for attr in dir(constraint):
                if not attr.startswith("_") and not callable(getattr(constraint, attr)):
                    try:
                        setattr(new_constraint, attr, getattr(constraint, attr))
                    except AttributeError:
                        pass
            
            # Handle targets for ARMATURE constraint type
            if constraint.type == 'ARMATURE':
                # Ensure the new constraint has the correct number of targets
                while len(new_constraint.targets) < len(constraint.targets):
                    new_constraint.targets.new()

                for i, tgt in enumerate(constraint.targets):
                    if tgt.target is not None:
                        new_constraint.targets[i].target = tgt.target
                        new_constraint.targets[i].subtarget = tgt.subtarget
                        new_constraint.targets[i].weight = tgt.weight
            
                    
            if use_index:        
                # Move the new constraint to the correct index
                constraints = bone.constraints
                for i in range(len(constraints) - 1, constraint_index, -1):
                    constraints.move(i, i-1)
                    
    self.report({'INFO'}, f"Constraint '{constraint_name}' copied successfully")
    return {'FINISHED'}
    
class BONE_OP_constraints(bpy.types.Operator):
    bl_idname = "bone.const"
    bl_label = "BONE OP constraints"

    type: bpy.props.StringProperty(default="")
    direction: bpy.props.EnumProperty(items=[('UP', "Up", ""), ('DOWN', "Down", "")])
    
    def execute(self, context):
        
        bone = context.active_pose_bone
        bone_index = bone.Bone_Constraint_list_index
        bone_list = bone.Bone_Constraint_list
        
        if self.type == "ADD_CONST":
            pass
        if self.type == "X_CONST":
            
            if 0 <= bone_index < len(bone_list):
                cual = bone_list[bone_index]

                bpy.ops.constraint.delete(constraint=cual.name, owner='BONE')
                
                if bone.Bone_Constraint_list_index < 1:
                    bone.Bone_Constraint_list_index = 0
                else:
                    bone.Bone_Constraint_list_index = bone_index-1
                    
        if self.type == "DEL_ALL":
            if bone:
                for constraint in bone.constraints:
                    bone.constraints.remove(constraint)
        
        if self.type == "NONE":         
            if bone:

                if 0 <= bone_index < len(bone_list):
                    cual = bone_list[bone_index]
                    if self.direction == 'UP':
                        if bone_index > 0:
                            bpy.ops.constraint.move_to_index(constraint=cual.name, owner='BONE', index=bone_index-1)
                            bone.Bone_Constraint_list_index = bone_index-1
                    else:
                        if bone_index < len(bone.Bone_Constraint_list)-1:
                            bpy.ops.constraint.move_to_index(constraint=cual.name, owner='BONE', index=bone_index+1)
                            bone.Bone_Constraint_list_index = bone_index+1
        
        if 0 <= bone_index < len(bone_list):
            cual = bone_list[bone_index]

            if self.type == "DUPLICATE_CONST":
                bpy.ops.constraint.copy(constraint=cual.name, owner='BONE')
                self.report({'INFO'}, f"Constraint '{cual.name}' Duplicate")
            if self.type == "APPLY_CONST":
                bpy.ops.constraint.apply(constraint=cual.name, owner='BONE')
                self.report({'INFO'}, f"Constraint '{cual.name}' Apply")
                if bone_index > 0:
                    bone.Bone_Constraint_list_index = bone_index-1
            if self.type == "COPY_SEL_CONST":
                copy_cosntraint(self, context, cual.name, use_index=False)
            if self.type == "COPY_SEL_CONST_INDEX":
                copy_cosntraint(self, context, cual.name, use_index=True)
                
        

                
        return {'FINISHED'}
    
@bpy.app.handlers.persistent   
def Bone_Const_Actulidor(dummy):
    obj = bpy.context.object
    bone = bpy.context.active_pose_bone
    
    
    if bone is not None:
        Bone_Constraint_list_index = bone.Bone_Constraint_list_index
        Bone_Constraint_list = bone.Bone_Constraint_list
        cuanto = len(Bone_Constraint_list)
        # Clear the current custom constraint list
        Bone_Constraint_list.clear()

        # Iterate through the actual constraints of the bone
        for constraint in bone.constraints:
            new_constraint = Bone_Constraint_list.add()
            new_constraint.name = constraint.name 
            new_constraint.type = constraint.type
            for icon in constraint_types:
                if constraint.type == icon[0]:
                    new_constraint.icon = icon[2]
            
        if cuanto != len(Bone_Constraint_list):
            if not cuanto > len(Bone_Constraint_list):
                bone.Bone_Constraint_list_index = (cuanto)
  
bpy.app.handlers.depsgraph_update_pre.append(Bone_Const_Actulidor)
    
def template_constraints(layout, context, Bone_Constraint_list_index, Bone_Constraint_list):
    obj = context.object
    if 0 <= Bone_Constraint_list_index < len(Bone_Constraint_list):
        props = Bone_Constraint_list[Bone_Constraint_list_index]
        if context.active_pose_bone:
            bone = context.active_pose_bone
            for constraint in bone.constraints:
                if props.name == constraint.name:
                    layout.prop(constraint, "target")
                    layout.prop(constraint, "subtarget")

                    layout.prop(constraint, "influence")
