import bpy 
import time
import sys
import os
from bl_ui.generic_ui_list import draw_ui_list
current_dir = os.path.dirname(__file__)
sys.path.append(current_dir) 

from Operadores.Rig_Creator_Operation.RIg_Constraint_G import *
from Paneles.Paneles_Constraint.Constraint import *

class ModifierButtonsPanel:
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "modifier"
    bl_options = {'HIDE_HEADER'}


class DATA_PT_modifiers(ModifierButtonsPanel, Panel):
    bl_label = "Modifiers"

    @classmethod
    def poll(cls, context):
        ob = context.object
        return (ob and ob.type != 'GPENCIL' and context.space_data.context == 'MODIFIER')

    def draw(self, _context):
        layout = self.layout
        layout.operator("wm.call_menu", text="Add Modifier", icon='ADD').name = "OBJECT_MT_modifier_add"
        layout.template_modifiers()



class OBJECT_CONST_PropertyGroup(bpy.types.PropertyGroup):
    
    name: bpy.props.StringProperty(default = "")
    icon: bpy.props.StringProperty(default = "CONSTRAINT_BONE")
    type: bpy.props.StringProperty(default = "")
    
class OBJECT_CONST_UL_List(bpy.types.UIList):
    
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            if context.active_object:
                Obj = context.active_object
                for constraint in Obj.constraints:
                    if constraint.name == item.name:
                        row = layout.row(align=True)
                        row.alignment = 'LEFT'
                        row.scale_x = 1.1
                        row.prop(constraint, "name", text="", icon=item.icon,  emboss=False)
                        row = layout.row(align=True)
                        row.alignment = 'RIGHT'
                        row.prop(constraint, "influence", text="",  emboss=False)
                        row.prop(constraint, "enabled", text="",icon='HIDE_ON',  emboss=False)
                
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon=item.icon)
            
class ObjectConstraintPanel:
    bl_context = "constraint"

    @classmethod
    def poll(cls, context):
        ob = context.object
        return (ob and context.space_data.context == 'CONSTRAINT')
           
class OBJECT_PT_constraints(ObjectConstraintPanel, Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_label = "Object Constraints"
    bl_options = {'HIDE_HEADER'}

    def draw(self, _context):
        layout = self.layout
        layout.operator_menu_enum("object.constraint_add", "type", text="Add Object Constraint")
        layout.template_constraints(use_bone_constraints=False)
        
class OBJECT_Constraint_list(bpy.types.Panel):
    bl_idname = "OBJECT_Constraint_list"
    bl_label = "Constraint List"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "constraint"
    bl_order = 0
    
    @classmethod
    def poll(cls, context):
        ob = context.object
        return (ob and context.space_data.context == 'CONSTRAINT')
    
    def draw(self, _context):
        layout = self.layout
        Constraint_list_Draw(self, _context, use_Bone=False)
 
class Copy_Constraint_To_Active_OBJ(bpy.types.Menu):
    bl_idname = "Copy_Constraint_To_Active_OBJ"
    bl_label = "Copy Constraint To Active OBJ"
    
    def draw(self, context):
        layout = self.layout
        Extra_Botones(self, context, use_Bone = False)
             
def Obj_copy_constraint(self, context, constraint_name, use_index=True):
    source_obj = context.object

    # Ensure we have an active object
    if source_obj is None:
        self.report({'ERROR'}, "No active object found")
        return {'CANCELLED'}

    # Find the constraint in the active object
    constraint = None
    for const in source_obj.constraints:
        if const.name == constraint_name:
            constraint = const
            break
    if not constraint:
        self.report({'ERROR'}, f"Constraint '{constraint_name}' not found")
        return {'CANCELLED'}
    
    # Get the index of the constraint
    if use_index:
        constraint_index = source_obj.constraints.find(constraint_name)
    
    # Copy the constraint to the selected objects
    selected_objects = [obj for obj in context.selected_objects if obj != source_obj]
    
    if not selected_objects:
        self.report({'ERROR'}, "No other selected objects found")
        return {'CANCELLED'}

    for target_obj in selected_objects:
        # Copy the constraint to the target object
        new_constraint = target_obj.constraints.new(type=constraint.type)
        for attr in dir(constraint):
            if not attr.startswith("_") and not callable(getattr(constraint, attr)):
                try:
                    setattr(new_constraint, attr, getattr(constraint, attr))
                except AttributeError:
                    pass
                
        # Handle targets for ARMATURE constraint type
        if constraint.type == 'ARMATURE':
            # Ensure the new constraint has the correct number of targets
            while len(new_constraint.targets) < len(constraint.targets):
                new_constraint.targets.new()

            for i, tgt in enumerate(constraint.targets):
                if tgt.target is not None:
                    new_constraint.targets[i].target = tgt.target
                    new_constraint.targets[i].subtarget = tgt.subtarget
                    new_constraint.targets[i].weight = tgt.weight
                    
        # Move the new constraint to the correct index
        if use_index:
            constraints = target_obj.constraints
            for i in range(len(constraints) - 1, constraint_index, -1):
                constraints.move(i, i-1)
    
    self.report({'INFO'}, "Constraints copied successfully")
    return {'FINISHED'}

class OBJECT_OP_constraints(bpy.types.Operator):
    bl_idname = "object.const"
    bl_label = "OBJECT OP constraints"

    type: bpy.props.StringProperty(default="")
    direction: bpy.props.EnumProperty(items=[('UP', "Up", ""), ('DOWN', "Down", "")])
    
    def execute(self, context):
        
        obj = context.active_object
        obj_index = obj.Object_Constraint_list_index
        obj_list = obj.Object_Constraint_list
        
        if self.type == "ADD_CONST":
            pass
        if self.type == "X_CONST":
            
            if 0 <= obj_index < len(obj_list):
                cual = obj_list[obj_index]

                bpy.ops.constraint.delete(constraint=cual.name, owner='OBJECT')
                
                if obj.Object_Constraint_list_index < 1:
                    obj.Object_Constraint_list_index = 0
                else:
                    obj.Object_Constraint_list_index = obj_index-1
        if self.type == "DEL_ALL":
            if obj:
                for constraint in obj.constraints:
                    obj.constraints.remove(constraint)          
        if self.type == "NONE":         
            if obj:

                if 0 <= obj_index < len(obj_list):
                    cual = obj_list[obj_index]
                    if self.direction == 'UP':
                        if obj_index > 0:
                            bpy.ops.constraint.move_to_index(constraint=cual.name, owner='OBJECT', index=obj_index-1)
                            obj.Object_Constraint_list_index = obj_index-1
                    else:
                        if obj_index < len(obj.Object_Constraint_list)-1:
                            bpy.ops.constraint.move_to_index(constraint=cual.name, owner='OBJECT', index=obj_index+1)
                            obj.Object_Constraint_list_index = obj_index+1
        
        if 0 <= obj_index < len(obj_list):
            cual = obj_list[obj_index]

            if self.type == "DUPLICATE_CONST":
                bpy.ops.constraint.copy(constraint=cual.name, owner='OBJECT')
            if self.type == "APPLY_CONST":
                bpy.ops.constraint.apply(constraint=cual.name, owner='OBJECT')
                if obj_index > 0:
                    obj.Object_Constraint_list_index = obj_index-1
            if self.type == "COPY_SEL_CONST":
                Obj_copy_constraint(self, context, cual.name, use_index = False)
            if self.type == "COPY_SEL_CONST_INDEX":
                Obj_copy_constraint(self, context, cual.name, use_index = True)
                  
        return {'FINISHED'}
    
@bpy.app.handlers.persistent   
def Obj_Const_Actulidor(dummy):
    obj = bpy.context.active_object
    
    if obj is not None:
        obj_Constraint_list = obj.Object_Constraint_list
        cuanto = len(obj_Constraint_list)
        # Clear the current custom constraint list
        obj_Constraint_list.clear()

        # Iterate through the actual constraints of the bone
        for constraint in obj.constraints:
            new_constraint = obj_Constraint_list.add()
            new_constraint.name = constraint.name 
            new_constraint.type = constraint.type
            for icon in constraint_types:
                if constraint.type == icon[0]:
                    new_constraint.icon = icon[2]
            
        if cuanto != len(obj_Constraint_list):
            if not cuanto > len(obj_Constraint_list):
                obj.Object_Constraint_list_index = (cuanto)
                  
    
bpy.app.handlers.depsgraph_update_pre.append(Obj_Const_Actulidor)