import bpy
import sys
import os
current_dir = os.path.dirname(__file__)
sys.path.append(current_dir)

from Def_save.List_custom import *

def duplicado(obj):
    bone_names = set()
    for idx, item in enumerate(obj.Rig_select_bones):
        if item.bone in bone_names:
            return True
        bone_names.add(item.bone)
    return False


class deselect_bones_list(bpy.types.UIList):
    
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            
            def icon_():
                if not item.active:
                    return "CHECKBOX_DEHLT"
                else:
                    return "CHECKBOX_HLT"
                
            def icon_1():
                if not item.candado:
                    return "UNLOCKED"
                else:
                    return "LOCKED"
                
            obj= context.active_object
            if obj:
                bone = obj.data.bones.get(item.bone)
                layout.active = item.active
                if bone:
                    layout.label(text=item.name,icon="BONE_DATA")
                    row = layout.row(align=True)
                    row.prop(item, "active",text="", emboss=False, icon=icon_())
                    row.prop(item, "candado",text="", emboss=False, icon=icon_1())
                else:
                    layout.label(text="NONE", icon="GROUP_BONE")
            else:
                layout.label(text="ERROR")
            
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="")


class deselect_bones_props(bpy.types.PropertyGroup):
    
    bone: bpy.props.StringProperty()
    active: bpy.props.BoolProperty(default=True)
    candado: bpy.props.BoolProperty(default=False)
    mode:bpy.props.EnumProperty(
        items=[("EDIT", "Edit Mode", ""),
               ("POSE", "Pose Mode", ""),
               ("ALL", "All Mode", ""),
               ],
        default='ALL'
    )
    invert: bpy.props.BoolProperty(default=False)
    

class deselect_bones_panel(bpy.types.Panel):
    bl_idname = "select_bones_panel"
    bl_label = "Sel/Deselect Auto"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "data"
    bl_parent_id = "DATA_PT_pose"
    bl_options = {'DEFAULT_CLOSED'}
    
    @classmethod
    def poll(cls, context):
        return (
            context.active_object and context.active_object.type == 'ARMATURE')
        
    def draw(self, context):
        layout = self.layout
        
        
        obj= context.active_object
        row = layout.row()
        
        list = obj.Rig_select_bones
        indext = obj.Rig_select_bones_index
        
        draw_base_list(
                row,
                "deselect_bones_list",
                obj,
                "Rig_select_bones",
                "Rig_select_bones_index",
                obj.Rig_select_bones,
                obj.Rig_select_bones_index,
                "select.bones_rig",
                op_type=[
                    "ADD","REMOVE",
                    "MOVE_UP","MOVE_DOWN"
                    ],
                use_menu=True,
                menu= "deselect._bone_menu"
            )

        if list:
            
            prop = list[indext]
            col = layout.column()
            
            col.alignment = "RIGHT"
            col_ = col.column()
            col_.alert = duplicado(obj)
            col_.prop_search(prop, "bone", obj.data, "bones", text="", icon="BONE_DATA")
            if duplicado(obj):
                col_.label(text="Bone Duplicator", icon="ERROR")
            col.prop(prop, "mode", text="")
            col.prop(prop, "invert", text=f"Select ({prop.invert})")

class deselect_bone_menu(bpy.types.Menu):
    bl_idname = "deselect._bone_menu"
    bl_label = ""

    def draw(self, context):
        layout = self.layout
        layout.operator("select.bones_rig", text="Clear").type = "CLEAR"
        layout.operator("select.bones_rig", text="Clear Unlocked").type = "CLEAR_LOCKED"  
            
class deselect_bones_operation(bpy.types.Operator):
    bl_idname = "select.bones_rig"
    bl_label = ""

    type: bpy.props.StringProperty()

    
    def execute(self, context):
        obj = context.object
        
        def move(delta):
            lists = obj.Rig_select_bones
            index = obj.Rig_select_bones_index
            
            if index < len(lists):
                list = lists[index]
                
                idx = index
                # Ensure idx is within valid bounds
                if idx > len(lists) - 1:
                    idx = len(lists) - 1
                elif idx < 0:
                    idx = 0

                new_idx = idx + delta

                # Ensure new_idx is within valid bounds
                if new_idx < 0:
                    new_idx = 0
                elif new_idx > len(lists) - 1:
                    new_idx = len(lists) - 1

                obj.Rig_select_bones.move(idx, new_idx)
                obj.Rig_select_bones_index = new_idx
        
        
        if self.type == "ADD":
            
            bones = None
            if obj.mode == "POSE":
                bones = context.selected_pose_bones
            elif obj.mode == "EDIT":
                bones = context.selected_editable_bones
                
            if bones:   
                for bone in bones:
                    bone_name = bone.name
                    existing_bone_index = None      
                    # Check if the bone name already exists in the collection
                    for idx, item in enumerate(obj.Rig_select_bones):
                        if item.bone == bone_name:
                            existing_bone_index = idx
                            break
                        
                    if existing_bone_index is None:
                        # If the bone name does not exist, add it to the collection
                        new = obj.Rig_select_bones.add()
                        new.bone = bone_name
                        new.name = bone_name
                        obj.Rig_select_bones_index = len(obj.Rig_select_bones) - 1
                    else:
                        # If the bone name exists, set the index to the existing one
                        obj.Rig_select_bones_index = existing_bone_index

                    #obj.Rig_select_bones_index = existing_bone_index
            else:
                new = obj.Rig_select_bones.add()
                new.bone = "NONE"
                new.name = "NONEP"
                obj.Rig_select_bones_index = len(obj.Rig_select_bones) - 1
                
        elif self.type == "REMOVE":
            list = obj.Rig_select_bones
            index = obj.Rig_select_bones_index
            list.remove(index)     
            obj.Rig_select_bones_index = min(max(0, index - 1), len(list) - 1)
            
        elif self.type == "CLEAR":
            list = obj.Rig_select_bones
            list.clear()
        
        elif self.type == "CLEAR_LOCKED":
            lists = obj.Rig_select_bones
            indices_to_remove = []

            # Collect indices to remove first to avoid modifying the list during iteration
            for idx, item in enumerate(lists):
                if not item.candado:
                    indices_to_remove.append(idx)

            # Remove items from the list in reverse order of their indices
            for idx in reversed(indices_to_remove):
                lists.remove(idx)   
            
            obj.Rig_select_bones_index = len(obj.Rig_select_bones) - 1
            
        elif self.type == "MOVE_UP":
            move(-1)
        elif self.type == "MOVE_DOWN":
            move(1)    
            
        return {"FINISHED"}
    
@bpy.app.handlers.persistent            
def deselect_bones_rig_(dummy):
    obj = bpy.context.active_object
    if obj and obj.type == 'ARMATURE' :
        lists = obj.Rig_select_bones
        index = obj.Rig_select_bones_index
        
        def pose_deselect(list):
            bone = obj.pose.bones.get(list.bone)
            if bone:
                
                bone.bone.select = list.invert
                if list.invert == False:
                    if obj.data.bones.active == bone.bone:
                        obj.data.bones.active = None
                    
        def edit_deselect(list):
            bone = obj.data.edit_bones.get(list.bone)
            if bone:
                bone.select = list.invert
                if list.invert == False:
                    if obj.data.edit_bones.active == bone:
                        obj.data.edit_bones.active = None
        
        for list in lists:
            
            #actulizar nombre
            if list.name != list.bone:
                list.name = list.bone

            if list.active:
                if list.mode == 'POSE':
                    pose_deselect(list)
                elif list.mode == "EDIT":
                   edit_deselect(list)
                else:
                    if obj.mode == "POSE":
                        pose_deselect(list)
                    elif obj.mode == "EDIT":
                        edit_deselect(list)
        
          
                    
bpy.app.handlers.depsgraph_update_pre.append(deselect_bones_rig_) 
     
classes =(
    deselect_bone_menu,
    deselect_bones_list,
    deselect_bones_operation,
    deselect_bones_panel,
    deselect_bones_props,
    
)    
        
def regselect_bones():
    for cls in classes:
        bpy.utils.register_class(cls)

    bpy.types.Object.Rig_select_bones = bpy.props.CollectionProperty(type=deselect_bones_props) 
    bpy.types.Object.Rig_select_bones_index = bpy.props.IntProperty()
    
def unregselect_bones():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    
    del bpy.types.Object.Rig_select_bones 
    del bpy.types.Object.Rig_select_bones_index
